@@ -18,12 +18,14 @@
 ;;; along with GNU Guix.  If not, see <http://www.gnu.org/licenses/>.
 
 (define-module (gnu packages base)
-  #:use-module (guix licenses)
+  #:use-module ((guix licenses)
+                #:select (gpl3+ lgpl2.0+ public-domain))
   #:use-module (gnu packages)
   #:use-module (gnu packages acl)
   #:use-module (gnu packages bash)
   #:use-module (gnu packages bootstrap)
   #:use-module (gnu packages compression)
+  #:use-module (gnu packages gcc)
   #:use-module (gnu packages gawk)
   #:use-module (gnu packages guile)
   #:use-module (gnu packages multiprecision)
@@ -60,7 +62,7 @@
                   ,(string-append "--with-gawk="  ; for illustration purposes
                                  (assoc-ref %build-inputs "gawk")))))
    (inputs `(("gawk" ,gawk)))
-   (synopsis "GNU Hello")
+   (synopsis "Hello, GNU world: An example GNU package")
    (description "Yeah...")
    (home-page "http://www.gnu.org/software/hello/")
    (license gpl3+)))
@@ -77,7 +79,7 @@
              (base32
               "1qbjb1l7f9blckc5pqy8jlf6482hpx4awn2acmhyf5mv9wfq03p7"))))
    (build-system gnu-build-system)
-   (synopsis "GNU implementation of the Unix grep command")
+   (synopsis "Print lines matching a pattern")
    (description
     "The grep command searches one or more input files for lines containing a
 match to a specified pattern.  By default, grep prints the matching
@@ -88,16 +90,16 @@ lines.")
 (define-public sed
   (package
    (name "sed")
-   (version "4.2.1")
+   (version "4.2.2")
    (source (origin
             (method url-fetch)
             (uri (string-append "mirror://gnu/sed/sed-" version
                                 ".tar.bz2"))
             (sha256
              (base32
-              "13wlsb4sf5d5a82xjhxqmdvrrn36rmw5f0pl9qyb9zkvldnb7hra"))))
+              "1myvrmh99jsvk7v3d7crm0gcrq51hmmm1r2kjyyci152in1x2j7h"))))
    (build-system gnu-build-system)
-   (synopsis "GNU sed, a batch stream editor")
+   (synopsis "Stream editor")
    (arguments
     `(#:phases (alist-cons-before
                 'patch-source-shebangs 'patch-test-suite
@@ -133,7 +135,7 @@ substituting multiple occurrences of a string within a file.")
    (inputs `(("patch/gets" ,(search-patch "tar-gets-undeclared.patch"))))
    (arguments
     `(#:patches (list (assoc-ref %build-inputs "patch/gets"))))
-   (synopsis "GNU implementation of the `tar' archiver")
+   (synopsis "Managing tar archives")
    (description
     "The Tar program provides the ability to create tar archives, as well as
 various other kinds of manipulation.  For example, you can use Tar on
@@ -151,22 +153,22 @@ files (as archives).")
 (define-public patch
   (package
    (name "patch")
-   (version "2.6.1")
+   (version "2.7.1")
    (source (origin
             (method url-fetch)
             (uri (string-append "mirror://gnu/patch/patch-"
                                 version ".tar.xz"))
             (sha256
              (base32
-              "18012gxs9wc96izskp1q7bclrwns6rdmkn4jj31c8jbyfz6l5npq"))))
+              "1sqckf560pzwgniy00vcpdv2c9c11s4cmhlm14yqgg8avd3bl94i"))))
    (build-system gnu-build-system)
    (native-inputs '())                      ; FIXME: needs `ed' for the tests
    (arguments
-    (case-lambda
-      ((system) '(#:tests? #f))
-      ((system cross-system)
-       '(#:configure-flags '("ac_cv_func_strnlen_working=yes")))))
-   (synopsis "GNU Patch, a program to apply differences to files")
+    '(#:tests? #f)
+    ;; TODO: When cross-compiling, add this:
+    ;;  '(#:configure-flags '("ac_cv_func_strnlen_working=yes"))
+    )
+   (synopsis "Apply differences to originals, with optional backups")
    (description
     "GNU Patch takes a patch file containing a difference listing produced by
 the diff program and applies those differences to one or more original files,
@@ -177,19 +179,16 @@ producing patched versions.")
 (define-public diffutils
   (package
    (name "diffutils")
-   (version "3.2")
+   (version "3.3")
    (source (origin
             (method url-fetch)
             (uri (string-append "mirror://gnu/diffutils/diffutils-"
                                 version ".tar.xz"))
             (sha256
              (base32
-              "0jci0wv68025xd0s0rq4s5qxpx56dd9d730lka63qpzk1rfvfkxb"))))
+              "1761vymxbp4wb5rzjvabhdkskk95pghnn67464byvzb5mfl8jpm2"))))
    (build-system gnu-build-system)
-   (inputs `(("patch/gets"
-              ,(search-patch "diffutils-gets-undeclared.patch"))))
-   (arguments `(#:patches (list (assoc-ref %build-inputs "patch/gets"))))
-   (synopsis "Programs to find differences among text files")
+   (synopsis "Comparing and merging files")
    (description
     "GNU Diffutils is a package of several programs related to finding
 differences between files.
@@ -235,16 +234,14 @@ You can use the sdiff command to merge two files interactively.")
     `(("patch/absolute-paths"
        ,(search-patch "findutils-absolute-paths.patch"))))
    (arguments
-    (case-lambda
-      ((system)
-       `(#:patches (list (assoc-ref %build-inputs "patch/absolute-paths"))))
-      ((system cross-system)
-       ;; Work around cross-compilation failure.
-       ;; See <http://savannah.gnu.org/bugs/?27299#comment1>.
-       `(#:configure-flags '("gl_cv_func_wcwidth_works=yes")
-         ,@(arguments cross-system)))))
-   (synopsis "Basic directory searching utilities of the GNU operating
-system")
+    `(#:patches (list (assoc-ref %build-inputs "patch/absolute-paths")))
+
+    ;; TODO: Work around cross-compilation failure.
+    ;; See <http://savannah.gnu.org/bugs/?27299#comment1>.
+    ;; `(#:configure-flags '("gl_cv_func_wcwidth_works=yes")
+    ;;   ,@(arguments cross-system))
+    )
+   (synopsis "Operating on files matching given criteria")
    (description
     "The GNU Find Utilities are the basic directory searching utilities of
 the GNU operating system.  These programs are typically used in conjunction
@@ -264,14 +261,14 @@ The tools supplied with this package are:
 (define-public coreutils
   (package
    (name "coreutils")
-   (version "8.20")
+   (version "8.21")
    (source (origin
             (method url-fetch)
             (uri (string-append "mirror://gnu/coreutils/coreutils-"
                                 version ".tar.xz"))
             (sha256
              (base32
-              "1cly97xdy3v4nbbx631k43smqw0nnpn651kkprs0yyl2cj3pkjyv"))))
+              "064f512185iysqqcvhnhaf3bfmzrvcgs7n405qsyp99zmfyl9amd"))))
    (build-system gnu-build-system)
    (inputs `(("acl"  ,acl)
              ("gmp"  ,gmp)
@@ -291,9 +288,7 @@ The tools supplied with this package are:
                       (("#!/bin/sh")
                        (format #f "#!~a/bin/bash" bash)))))
                 %standard-phases)))
-   (synopsis
-    "The basic file, shell and text manipulation utilities of the GNU
-operating system")
+   (synopsis "Core GNU utilities (file, text, shell)")
    (description
     "The GNU Core Utilities are the basic file, shell and text manipulation
 utilities of the GNU operating system.  These are the core utilities which
@@ -327,8 +322,7 @@ are expected to exist on every operating system.")
                        (format #f "default_shell[] = \"~a/bin/bash\";\n"
                                bash)))))
                 %standard-phases)))
-   (synopsis "GNU Make, a program controlling the generation of non-source
-files from sources")
+   (synopsis "Remake files automatically")
    (description
     "Make is a tool which controls the generation of executables and other
 non-source files of a program from the program's source files.
@@ -343,14 +337,14 @@ that it is possible to use Make to build and install the program.")
 (define-public binutils
   (package
    (name "binutils")
-   (version "2.22")
+   (version "2.23.2")
    (source (origin
             (method url-fetch)
             (uri (string-append "mirror://gnu/binutils/binutils-"
                                 version ".tar.bz2"))
             (sha256
              (base32
-              "1a9w66v5dwvbnawshjwqcgz7km6kw6ihkzp6sswv9ycc3knzhykc"))))
+              "15qhbkz3r266xaa52slh857qn3abw7rb2x2jnhpfrafpzrb4x4gy"))))
    (build-system gnu-build-system)
 
    ;; Split Binutils in several outputs, mostly to avoid collisions in
@@ -368,10 +362,13 @@ that it is possible to use Make to build and install the program.")
                           "LDFLAGS=-static-libgcc"
 
                           ;; Don't search under /usr/lib & co.
-                          "--with-lib-path=/no-ld-lib-path")))
+                          "--with-lib-path=/no-ld-lib-path"
 
-   (synopsis "GNU Binutils, tools for manipulating binaries (linker,
-assembler, etc.)")
+                          ;; Glibc 2.17 has a "comparison of unsigned
+                          ;; expression >= 0 is always true" in wchar.h.
+                          "--disable-werror")))
+
+   (synopsis "Binary utilities: bfd gas gprof ld")
    (description
     "The GNU Binutils are a collection of binary tools.  The main ones are
 `ld' (the GNU linker) and `as' (the GNU assembler).  They also include the
@@ -379,120 +376,6 @@ BFD (Binary File Descriptor) library, `gprof', `nm', `strip', etc.")
    (license gpl3+)
    (home-page "http://www.gnu.org/software/binutils/")))
 
-(define-public gcc-4.7
-  (let ((stripped? #t))                         ; TODO: make this a parameter
-    (package
-     (name "gcc")
-     (version "4.7.2")
-     (source (origin
-              (method url-fetch)
-              (uri (string-append "mirror://gnu/gcc/gcc-"
-                                  version "/gcc-" version ".tar.bz2"))
-              (sha256
-               (base32
-                "115h03hil99ljig8lkrq4qk426awmzh0g99wrrggxf8g07bq74la"))))
-     (build-system gnu-build-system)
-     (inputs `(("gmp" ,gmp)
-               ("mpfr" ,mpfr)
-               ("mpc" ,mpc)))           ; TODO: libelf, ppl, cloog, zlib, etc.
-     (arguments
-      (lambda (system)
-        `(#:out-of-source? #t
-          #:strip-binaries? ,stripped?
-          #:configure-flags
-          `("--enable-plugin"
-            "--enable-languages=c,c++"
-            "--disable-multilib"
-
-            "--with-local-prefix=/no-gcc-local-prefix"
-
-            ,(let ((libc (assoc-ref %build-inputs "libc")))
-               (if libc
-                   (string-append "--with-native-system-header-dir=" libc
-                                  "/include")
-                   "--without-headers")))
-          #:make-flags
-          (let ((libc (assoc-ref %build-inputs "libc")))
-            `(,@(if libc
-                    (list (string-append "LDFLAGS_FOR_BUILD="
-                                         "-L" libc "/lib "
-                                         "-Wl,-dynamic-linker "
-                                         "-Wl," libc
-                                         ,(glibc-dynamic-linker system)))
-                    '())
-              ,(string-append "BOOT_CFLAGS=-O2 "
-                              ,(if stripped? "-g0" "-g"))))
-
-          #:tests? #f
-          #:phases
-          (alist-cons-before
-           'configure 'pre-configure
-           (lambda* (#:key inputs outputs #:allow-other-keys)
-             (let ((out  (assoc-ref outputs "out"))
-                   (libc (assoc-ref inputs "libc")))
-               (when libc
-                 ;; The following is not performed for `--without-headers'
-                 ;; cross-compiler builds.
-
-                 ;; Fix the dynamic linker's file name.
-                 (substitute* (find-files "gcc/config"
-                                          "^linux(64|-elf)?\\.h$")
-                   (("#define GLIBC_DYNAMIC_LINKER([^ ]*).*$" _ suffix)
-                    (format #f "#define GLIBC_DYNAMIC_LINKER~a \"~a\"~%"
-                            suffix
-                            (string-append libc ,(glibc-dynamic-linker system)))))
-
-                 ;; Tell where to find libstdc++, libc, and `?crt*.o', except
-                 ;; `crt{begin,end}.o', which come with GCC.
-                 (substitute* (find-files "gcc/config"
-                                          "^(gnu-user(64)?|linux-elf)\\.h$")
-                   (("#define LIB_SPEC (.*)$" _ suffix)
-                    ;; Note that with this "lib" spec, we may still add a
-                    ;; RUNPATH to GCC even when `libgcc_s' is not NEEDED.
-                    ;; There's not much that can be done to avoid it, though.
-                    (format #f "#define LIB_SPEC \"-L~a/lib %{!static:-rpath=~a/lib \
-%{!static-libgcc:-rpath=~a/lib64 -rpath=~a/lib}} \" ~a~%"
-                            libc libc out out suffix))
-                   (("#define STARTFILE_SPEC.*$" line)
-                    (format #f "#define STANDARD_STARTFILE_PREFIX_1 \"~a/lib\"
-#define STANDARD_STARTFILE_PREFIX_2 \"\"
-~a~%"
-                            libc line))))
-
-               ;; Don't retain a dependency on the build-time sed.
-               (substitute* "fixincludes/fixincl.x"
-                 (("static char const sed_cmd_z\\[\\] =.*;")
-                  "static char const sed_cmd_z[] = \"sed\";"))))
-
-           (alist-cons-after
-            'configure 'post-configure
-            (lambda _
-              ;; Don't store configure flags, to avoid retaining references to
-              ;; build-time dependencies---e.g., `--with-ppl=/nix/store/xxx'.
-              (substitute* "Makefile"
-                (("^TOPLEVEL_CONFIGURE_ARGUMENTS=(.*)$" _ rest)
-                 "TOPLEVEL_CONFIGURE_ARGUMENTS=\n")))
-            (alist-replace 'install
-                           (lambda* (#:key outputs #:allow-other-keys)
-                             (zero?
-                              (system* "make"
-                                       ,(if stripped?
-                                            "install-strip"
-                                            "install"))))
-                           %standard-phases))))))
-
-     (properties `((gcc-libc . ,(assoc-ref inputs "libc"))))
-     (synopsis "The GNU Compiler Collection")
-     (description
-      "The GNU Compiler Collection includes compiler front ends for C, C++,
-Objective-C, Fortran, OpenMP for C/C++/Fortran, Java, and Ada, as well as
-libraries for these languages (libstdc++, libgcj, libgomp,...).
-
-GCC development is a part of the GNU Project, aiming to improve the compiler
-used in the GNU system including the GNU/Linux variant.")
-     (license gpl3+)
-     (home-page "http://gcc.gnu.org/"))))
-
 (define-public glibc
   (package
    (name "glibc")
@@ -510,13 +393,28 @@ used in the GNU system including the GNU/Linux variant.")
    ;; users should automatically pull Linux headers as well.
    (propagated-inputs `(("linux-headers" ,linux-libre-headers)))
 
+   ;; Store the locales separately (~100 MiB).  Note that "out" retains a
+   ;; reference to them anyway, so there's no space savings here.
+   ;; TODO: Eventually we may want to add a $LOCALE_ARCHIVE search path like
+   ;; Nixpkgs does.
+   (outputs '("out" "locales"))
+
    (arguments
     `(#:out-of-source? #t
       #:patches (list (assoc-ref %build-inputs "patch/ld.so.cache"))
       #:configure-flags
       (list "--enable-add-ons"
             "--sysconfdir=/etc"
-            "--localedir=/var/run/current-system/sw/lib/locale" ; XXX
+            (string-append "--localedir=" (assoc-ref %outputs "locales")
+                           "/share/locale")
+
+            ;; `--localedir' is not honored, so work around it.
+            ;; See <http://sourceware.org/ml/libc-alpha/2013-03/msg00093.html>.
+            (string-append "libc_cv_localedir="
+                           (assoc-ref %outputs "locales")
+                           "/share/locale")
+
+
             (string-append "--with-headers="
                            (assoc-ref %build-inputs "linux-headers")
                            "/include")
@@ -582,7 +480,12 @@ used in the GNU system including the GNU/Linux variant.")
                     (substitute* "libio/iopopen.c"
                       (("/bin/sh")
                        (string-append out "/bin/bash")))))
-                %standard-phases)))
+                (alist-cons-after
+                 'install 'install-locales
+                 (lambda _
+                   (zero? (system* "make" "localedata/install-locales")))
+                 %standard-phases))))
+
    (inputs `(("patch/ld.so.cache"
               ,(search-patch "glibc-no-ld-so-cache.patch"))
              ("static-bash" ,(static-package bash-light))))
@@ -597,6 +500,74 @@ with the Linux kernel.")
    (license lgpl2.0+)
    (home-page "http://www.gnu.org/software/libc/")))
 
+(define-public tzdata
+  (package
+    (name "tzdata")
+    (version "2013c")
+    (source (origin
+             (method url-fetch)
+             (uri (string-append
+                   "http://www.iana.org/time-zones/repository/releases/tzdata"
+                   version ".tar.gz"))
+             (sha256
+              (base32
+               "11swq6fg20m2dh520qcr8vb23gqhzbvqhizx8wifnmci4gmsg5z5"))))
+    (build-system gnu-build-system)
+    (arguments
+     '(#:tests? #f
+       #:make-flags (let ((out (assoc-ref %outputs "out"))
+                          (tmp (getenv "TMPDIR")))
+                      (list (string-append "TOPDIR=" out)
+                            (string-append "TZDIR=" out "/share/zoneinfo")
+
+                            ;; Discard zic, dump, and tzselect, already
+                            ;; provided by glibc.
+                            (string-append "ETCDIR=" tmp "/etc")
+
+                            ;; Likewise for the C library routines.
+                            (string-append "LIBDIR=" tmp "/lib")
+                            (string-append "MANDIR=" tmp "/man")
+
+                            "AWK=awk"
+                            "CC=gcc"))
+       #:modules ((guix build utils)
+                  (guix build gnu-build-system)
+                  (srfi srfi-1))
+       #:phases
+       (alist-replace
+        'unpack
+        (lambda* (#:key inputs #:allow-other-keys)
+          (and (zero? (system* "tar" "xvf" (assoc-ref inputs "source")))
+               (zero? (system* "tar" "xvf" (assoc-ref inputs "tzcode")))))
+        (alist-cons-after
+         'install 'post-install
+         (lambda* (#:key outputs #:allow-other-keys)
+           ;; Move data in the right place.
+           (let ((out (assoc-ref outputs "out")))
+             (copy-recursively (string-append out "/share/zoneinfo-posix")
+                               (string-append out "/share/zoneinfo/posix"))
+             (copy-recursively (string-append out "/share/zoneinfo-leaps")
+                               (string-append out "/share/zoneinfo/right"))
+             (delete-file-recursively (string-append out "/share/zoneinfo-posix"))
+             (delete-file-recursively (string-append out "/share/zoneinfo-leaps"))))
+         (alist-delete 'configure %standard-phases)))))
+    (inputs `(("tzcode" ,(origin
+                          (method url-fetch)
+                          (uri (string-append
+                                "http://www.iana.org/time-zones/repository/releases/tzcode"
+                                version ".tar.gz"))
+                          (sha256
+                           (base32
+                            "1w6nkdwhi6k9llshp4baac1yj43jqf3apdf6n66i0wvjj8qyjvp4"))))))
+    (home-page "http://www.iana.org/time-zones")
+    (synopsis "Database of current and historical time zones")
+    (description "The Time Zone Database (often called tz or zoneinfo)
+contains code and data that represent the history of local time for many
+representative locations around the globe. It is updated periodically to
+reflect changes made by political bodies to time zone boundaries, UTC offsets,
+and daylight-saving rules.")
+    (license public-domain)))
+
 
 ;;;
 ;;; Bootstrap packages.
@@ -653,7 +624,8 @@ with the Linux kernel.")
     ("findutils" ,findutils-boot0)
     ,@%bootstrap-inputs))
 
-(define* (nix-system->gnu-triplet system #:optional (vendor "unknown"))
+(define* (nix-system->gnu-triplet
+          #:optional (system (%current-system)) (vendor "unknown"))
   "Return an a guess of the GNU triplet corresponding to Nix system
 identifier SYSTEM."
   (let* ((dash (string-index system #\-))
@@ -665,10 +637,10 @@ identifier SYSTEM."
                        "linux-gnu"
                        os))))
 
-(define boot-triplet
+(define* (boot-triplet #:optional (system (%current-system)))
   ;; Return the triplet used to create the cross toolchain needed in the
   ;; first bootstrapping stage.
-  (cut nix-system->gnu-triplet <> "guix"))
+  (nix-system->gnu-triplet system "guix"))
 
 ;; Following Linux From Scratch, build a cross-toolchain in stage 0.  That
 ;; toolchain actually targets the same OS and arch, but it has the advantage
@@ -680,12 +652,12 @@ identifier SYSTEM."
    (package (inherit binutils)
      (name "binutils-cross-boot0")
      (arguments
-      (lambda (system)
-        `(#:guile ,%bootstrap-guile
-          #:implicit-inputs? #f
-          ,@(substitute-keyword-arguments (package-arguments binutils)
-              ((#:configure-flags cf)
-               `(list ,(string-append "--target=" (boot-triplet system))))))))
+      `(#:guile ,%bootstrap-guile
+        #:implicit-inputs? #f
+        ,@(substitute-keyword-arguments (package-arguments binutils)
+            ((#:configure-flags cf)
+             `(cons ,(string-append "--target=" (boot-triplet))
+                    ,cf)))))
      (inputs %boot0-inputs))))
 
 (define gcc-boot0
@@ -693,82 +665,80 @@ identifier SYSTEM."
    (package (inherit gcc-4.7)
      (name "gcc-cross-boot0")
      (arguments
-      (lambda (system)
-        `(#:guile ,%bootstrap-guile
-          #:implicit-inputs? #f
-          #:modules ((guix build gnu-build-system)
-                     (guix build utils)
-                     (ice-9 regex)
-                     (srfi srfi-1)
-                     (srfi srfi-26))
-          ,@(substitute-keyword-arguments ((package-arguments gcc-4.7) system)
-              ((#:configure-flags flags)
-               `(append (list ,(string-append "--target="
-                                              (boot-triplet system))
+      `(#:guile ,%bootstrap-guile
+        #:implicit-inputs? #f
+        #:modules ((guix build gnu-build-system)
+                   (guix build utils)
+                   (ice-9 regex)
+                   (srfi srfi-1)
+                   (srfi srfi-26))
+        ,@(substitute-keyword-arguments (package-arguments gcc-4.7)
+            ((#:configure-flags flags)
+             `(append (list ,(string-append "--target=" (boot-triplet))
 
-                              ;; No libc yet.
-                              "--without-headers"
+                            ;; No libc yet.
+                            "--without-headers"
 
-                              ;; Disable features not needed at this stage.
-                              "--disable-shared"
-                              "--enable-languages=c"
-                              "--disable-libmudflap"
-                              "--disable-libgomp"
-                              "--disable-libssp"
-                              "--disable-libquadmath"
-                              "--disable-decimal-float")
-                        (remove (cut string-match "--enable-languages.*" <>)
-                                ,flags)))
-              ((#:phases phases)
-               `(alist-cons-after
-                 'unpack 'unpack-gmp&co
-                 (lambda* (#:key inputs #:allow-other-keys)
-                   (let ((gmp  (assoc-ref %build-inputs "gmp-source"))
-                         (mpfr (assoc-ref %build-inputs "mpfr-source"))
-                         (mpc  (assoc-ref %build-inputs "mpc-source")))
+                            ;; Disable features not needed at this stage.
+                            "--disable-shared"
+                            "--enable-languages=c"
+                            "--disable-libmudflap"
+                            "--disable-libgomp"
+                            "--disable-libssp"
+                            "--disable-libquadmath"
+                            "--disable-decimal-float")
+                      (remove (cut string-match "--enable-languages.*" <>)
+                              ,flags)))
+            ((#:phases phases)
+             `(alist-cons-after
+               'unpack 'unpack-gmp&co
+               (lambda* (#:key inputs #:allow-other-keys)
+                 (let ((gmp  (assoc-ref %build-inputs "gmp-source"))
+                       (mpfr (assoc-ref %build-inputs "mpfr-source"))
+                       (mpc  (assoc-ref %build-inputs "mpc-source")))
 
-                     ;; To reduce the set of pre-built bootstrap inputs, build
-                     ;; GMP & co. from GCC.
-                     (for-each (lambda (source)
-                                 (or (zero? (system* "tar" "xvf" source))
-                                     (error "failed to unpack tarball"
-                                            source)))
-                               (list gmp mpfr mpc))
+                   ;; To reduce the set of pre-built bootstrap inputs, build
+                   ;; GMP & co. from GCC.
+                   (for-each (lambda (source)
+                               (or (zero? (system* "tar" "xvf" source))
+                                   (error "failed to unpack tarball"
+                                          source)))
+                             (list gmp mpfr mpc))
 
-                     ;; Create symlinks like `gmp' -> `gmp-5.0.5'.
-                     ,@(map (lambda (lib)
-                              `(symlink ,(package-full-name lib)
-                                        ,(package-name lib)))
-                            (list gmp mpfr mpc))
+                   ;; Create symlinks like `gmp' -> `gmp-5.0.5'.
+                   ,@(map (lambda (lib)
+                            `(symlink ,(package-full-name lib)
+                                      ,(package-name lib)))
+                          (list gmp mpfr mpc))
 
-                     ;; MPFR headers/lib are found under $(MPFR)/src, but
-                     ;; `configure' wrongfully tells MPC too look under
-                     ;; $(MPFR), so fix that.
-                     (substitute* "configure"
-                       (("extra_mpc_mpfr_configure_flags(.+)--with-mpfr-include=([^/]+)/mpfr(.*)--with-mpfr-lib=([^ ]+)/mpfr"
-                         _ equals include middle lib)
-                        (string-append "extra_mpc_mpfr_configure_flags" equals
-                                       "--with-mpfr-include=" include
-                                       "/mpfr/src" middle
-                                       "--with-mpfr-lib=" lib
-                                       "/mpfr/src"))
-                       (("gmpinc='-I([^ ]+)/mpfr -I([^ ]+)/mpfr" _ a b)
-                        (string-append "gmpinc='-I" a "/mpfr/src "
-                                       "-I" b "/mpfr/src"))
-                       (("gmplibs='-L([^ ]+)/mpfr" _ a)
-                        (string-append "gmplibs='-L" a "/mpfr/src")))))
-                 (alist-cons-after
-                  'install 'symlink-libgcc_eh
-                  (lambda* (#:key outputs #:allow-other-keys)
-                    (let ((out (assoc-ref outputs "out")))
-                      ;; Glibc wants to link against libgcc_eh, so provide
-                      ;; it.
-                      (with-directory-excursion
-                          (string-append out "/lib/gcc/"
-                                         ,(boot-triplet system)
-                                         "/" ,(package-version gcc-4.7))
-                        (symlink "libgcc.a" "libgcc_eh.a"))))
-                  ,phases)))))))
+                   ;; MPFR headers/lib are found under $(MPFR)/src, but
+                   ;; `configure' wrongfully tells MPC too look under
+                   ;; $(MPFR), so fix that.
+                   (substitute* "configure"
+                     (("extra_mpc_mpfr_configure_flags(.+)--with-mpfr-include=([^/]+)/mpfr(.*)--with-mpfr-lib=([^ ]+)/mpfr"
+                       _ equals include middle lib)
+                      (string-append "extra_mpc_mpfr_configure_flags" equals
+                                     "--with-mpfr-include=" include
+                                     "/mpfr/src" middle
+                                     "--with-mpfr-lib=" lib
+                                     "/mpfr/src"))
+                     (("gmpinc='-I([^ ]+)/mpfr -I([^ ]+)/mpfr" _ a b)
+                      (string-append "gmpinc='-I" a "/mpfr/src "
+                                     "-I" b "/mpfr/src"))
+                     (("gmplibs='-L([^ ]+)/mpfr" _ a)
+                      (string-append "gmplibs='-L" a "/mpfr/src")))))
+               (alist-cons-after
+                'install 'symlink-libgcc_eh
+                (lambda* (#:key outputs #:allow-other-keys)
+                  (let ((out (assoc-ref outputs "out")))
+                    ;; Glibc wants to link against libgcc_eh, so provide
+                    ;; it.
+                    (with-directory-excursion
+                        (string-append out "/lib/gcc/"
+                                       ,(boot-triplet)
+                                       "/" ,(package-version gcc-4.7))
+                      (symlink "libgcc.a" "libgcc_eh.a"))))
+                ,phases))))))
 
      (inputs `(("gmp-source" ,(package-source gmp))
                ("mpfr-source" ,(package-source mpfr))
@@ -812,20 +782,19 @@ identifier SYSTEM."
    (package (inherit glibc)
      (name "glibc-intermediate")
      (arguments
-      (lambda (system)
-        `(#:guile ,%bootstrap-guile
-          #:implicit-inputs? #f
+      `(#:guile ,%bootstrap-guile
+        #:implicit-inputs? #f
 
-          ,@(substitute-keyword-arguments (package-arguments glibc)
-              ((#:configure-flags flags)
-               `(append (list ,(string-append "--host=" (boot-triplet system))
-                              ,(string-append "--build="
-                                              (nix-system->gnu-triplet system))
+        ,@(substitute-keyword-arguments (package-arguments glibc)
+            ((#:configure-flags flags)
+             `(append (list ,(string-append "--host=" (boot-triplet))
+                            ,(string-append "--build="
+                                            (nix-system->gnu-triplet))
 
-                              ;; Build Sun/ONC RPC support.  In particular,
-                              ;; install rpc/*.h.
-                              "--enable-obsolete-rpc")
-                        ,flags))))))
+                            ;; Build Sun/ONC RPC support.  In particular,
+                            ;; install rpc/*.h.
+                            "--enable-obsolete-rpc")
+                      ,flags)))))
      (propagated-inputs `(("linux-headers" ,linux-libre-headers-boot0)))
      (inputs
       `( ;; A native GCC is needed to build `cross-rpcgen'.
@@ -847,40 +816,39 @@ that makes it available under the native tool names."
     (source #f)
     (build-system trivial-build-system)
     (arguments
-     (lambda (system)
-       `(#:guile ,%bootstrap-guile
-         #:modules ((guix build utils))
-         #:builder (begin
-                     (use-modules (guix build utils))
+     `(#:guile ,%bootstrap-guile
+       #:modules ((guix build utils))
+       #:builder (begin
+                   (use-modules (guix build utils))
 
-                     (let* ((binutils (assoc-ref %build-inputs "binutils"))
-                            (gcc      (assoc-ref %build-inputs "gcc"))
-                            (libc     (assoc-ref %build-inputs "libc"))
-                            (bash     (assoc-ref %build-inputs "bash"))
-                            (out      (assoc-ref %outputs "out"))
-                            (bindir   (string-append out "/bin"))
-                            (triplet  ,(boot-triplet system)))
-                       (mkdir-p bindir)
-                       (with-directory-excursion bindir
-                         (for-each (lambda (tool)
-                                     (symlink (string-append binutils "/bin/"
-                                                             triplet "-" tool)
-                                              tool))
-                                   '("ar" "ranlib"))
+                   (let* ((binutils (assoc-ref %build-inputs "binutils"))
+                          (gcc      (assoc-ref %build-inputs "gcc"))
+                          (libc     (assoc-ref %build-inputs "libc"))
+                          (bash     (assoc-ref %build-inputs "bash"))
+                          (out      (assoc-ref %outputs "out"))
+                          (bindir   (string-append out "/bin"))
+                          (triplet  ,(boot-triplet)))
+                     (mkdir-p bindir)
+                     (with-directory-excursion bindir
+                       (for-each (lambda (tool)
+                                   (symlink (string-append binutils "/bin/"
+                                                           triplet "-" tool)
+                                            tool))
+                                 '("ar" "ranlib"))
 
-                         ;; GCC-BOOT0 is a libc-less cross-compiler, so it
-                         ;; needs to be told where to find the crt files and
-                         ;; the dynamic linker.
-                         (call-with-output-file "gcc"
-                           (lambda (p)
-                             (format p "#!~a/bin/bash
+                       ;; GCC-BOOT0 is a libc-less cross-compiler, so it
+                       ;; needs to be told where to find the crt files and
+                       ;; the dynamic linker.
+                       (call-with-output-file "gcc"
+                         (lambda (p)
+                           (format p "#!~a/bin/bash
 exec ~a/bin/~a-gcc -B~a/lib -Wl,-dynamic-linker -Wl,~a/~a \"$@\"~%"
-                                     bash
-                                     gcc triplet
-                                     libc libc
-                                     ,(glibc-dynamic-linker system))))
+                                   bash
+                                   gcc triplet
+                                   libc libc
+                                   ,(glibc-dynamic-linker))))
 
-                         (chmod "gcc" #o555)))))))
+                       (chmod "gcc" #o555))))))
     (native-inputs
      `(("binutils" ,binutils)
        ("gcc" ,gcc)
@@ -896,9 +864,8 @@ exec ~a/bin/~a-gcc -B~a/lib -Wl,-dynamic-linker -Wl,~a/~a \"$@\"~%"
                                   (car (assoc-ref %boot1-inputs "bash"))))
          (bash (package (inherit bash-light)
                  (arguments
-                  (lambda (system)
-                    `(#:guile ,%bootstrap-guile
-                      ,@(package-arguments bash-light)))))))
+                  `(#:guile ,%bootstrap-guile
+                    ,@(package-arguments bash-light))))))
     (package-with-bootstrap-guile
      (package-with-explicit-inputs (static-package bash)
                                    `(("gcc" ,gcc)
@@ -932,10 +899,9 @@ exec ~a/bin/~a-gcc -B~a/lib -Wl,-dynamic-linker -Wl,~a/~a \"$@\"~%"
   (package-with-bootstrap-guile
    (package (inherit binutils)
      (arguments
-      (lambda (system)
-        `(#:guile ,%bootstrap-guile
-                  #:implicit-inputs? #f
-                  ,@(package-arguments binutils))))
+      `(#:guile ,%bootstrap-guile
+        #:implicit-inputs? #f
+        ,@(package-arguments binutils)))
      (inputs %boot2-inputs))))
 
 (define-public gcc-final
@@ -943,23 +909,22 @@ exec ~a/bin/~a-gcc -B~a/lib -Wl,-dynamic-linker -Wl,~a/~a \"$@\"~%"
   (package (inherit gcc-boot0)
     (name "gcc")
     (arguments
-     (lambda (system)
-       `(#:guile ,%bootstrap-guile
-         #:implicit-inputs? #f
+     `(#:guile ,%bootstrap-guile
+       #:implicit-inputs? #f
 
-         ;; Build again GMP & co. within GCC's build process, because it's hard
-         ;; to do outside (because GCC-BOOT0 is a cross-compiler, and thus
-         ;; doesn't honor $LIBRARY_PATH, which breaks `gnu-build-system'.)
-         ,@(substitute-keyword-arguments ((package-arguments gcc-boot0) system)
-             ((#:configure-flags boot-flags)
-              (let loop ((args ((package-arguments gcc-4.7) system)))
-                (match args
-                  ((#:configure-flags normal-flags _ ...)
-                   normal-flags)
-                  ((_ rest ...)
-                   (loop rest)))))
-             ((#:phases phases)
-              `(alist-delete 'symlink-libgcc_eh ,phases))))))
+       ;; Build again GMP & co. within GCC's build process, because it's hard
+       ;; to do outside (because GCC-BOOT0 is a cross-compiler, and thus
+       ;; doesn't honor $LIBRARY_PATH, which breaks `gnu-build-system'.)
+       ,@(substitute-keyword-arguments (package-arguments gcc-boot0)
+           ((#:configure-flags boot-flags)
+            (let loop ((args (package-arguments gcc-4.7)))
+              (match args
+                ((#:configure-flags normal-flags _ ...)
+                 normal-flags)
+                ((_ rest ...)
+                 (loop rest)))))
+           ((#:phases phases)
+            `(alist-delete 'symlink-libgcc_eh ,phases)))))
 
     (inputs `(("gmp-source" ,(package-source gmp))
               ("mpfr-source" ,(package-source mpfr))
