@@ -1,5 +1,6 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2012 Nikita Karetnikov <nikita@karetnikov.org>
+;;; Copyright © 2013 Ludovic Courtès <ludo@gnu.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -25,22 +26,24 @@
 (define-public ed
   (package
     (name "ed")
-    (version "1.5")
-    (source
-     (origin
-      (method url-fetch)
-      (uri (string-append
-            "mirror://gnu/ed/ed-"
-            version
-            ".tar.gz"))
-      (sha256
-       (base32
-        "18gvhyhwpabmgv4lh21lg8vl3z7acdyhh2mr2kj9g75wksj39pcp"))))
+    (version "1.8")
+    (source (origin
+             (method url-fetch)
+             (uri (string-append "mirror://gnu/ed/ed-"
+                                 version ".tar.gz"))
+             (sha256
+              (base32
+               "0wvj190ky5i0gm0pilx9k75l6alyc6h5s14fm3dbk90y7g9kihb4"))))
     (build-system gnu-build-system)
-    (arguments '(#:configure-flags '("CC=gcc")))
+    (arguments
+     '(#:configure-flags '("CC=gcc")
+       #:phases (alist-cons-before 'patch-source-shebangs 'patch-test-suite
+                                   (lambda _
+                                     (substitute* "testsuite/check.sh"
+                                       (("/bin/sh") (which "sh"))))
+                                   %standard-phases)))
     (home-page "http://www.gnu.org/software/ed/")
-    (synopsis
-     "GNU ed, an implementation of the standard Unix editor")
+    (synopsis "Line-oriented text editor")
     (description
      "GNU ed is a line-oriented text editor.  It is used to create,
 display, modify and otherwise manipulate text files, both
