@@ -21,42 +21,37 @@
   #:use-module (gnu packages libsigsegv)
   #:use-module (guix packages)
   #:use-module (guix download)
-  #:use-module (guix utils)
   #:use-module (guix build-system gnu))
 
 (define-public gawk
   (package
    (name "gawk")
-   (version "4.0.0")
+   (version "4.0.2")
    (source (origin
             (method url-fetch)
             (uri (string-append "mirror://gnu/gawk/gawk-" version
-                                ".tar.bz2"))
+                                ".tar.xz"))
             (sha256
-             (base32 "0sss7rhpvizi2a88h6giv0i7w5h07s2fxkw3s6n1hqvcnhrfgbb0"))))
+             (base32 "04vd0axif762mf781pj3days6ilv2333b9zi9c50y5mma66g5q91"))))
    (build-system gnu-build-system)
    (arguments
-    (case-lambda
-      ((system)
-       `(#:parallel-tests? #f             ; test suite fails in parallel
+    `(#:parallel-tests? #f                ; test suite fails in parallel
 
-         ;; Work around test failure on Cygwin.
-         #:tests? ,(not (string=? system "i686-cygwin"))
+      ;; Work around test failure on Cygwin.
+      #:tests? ,(not (string=? (%current-system) "i686-cygwin"))
 
-         #:phases (alist-cons-before
-                   'configure 'set-shell-file-name
-                   (lambda* (#:key inputs #:allow-other-keys)
-                     ;; Refer to the right shell.
-                     (let ((bash (assoc-ref inputs "bash")))
-                       (substitute* "io.c"
-                         (("/bin/sh")
-                          (string-append bash "/bin/bash")))))
-                   %standard-phases)))
-      ((system cross-system)
-       '(#:parallel-tests? #f))))
+      #:phases (alist-cons-before
+                'configure 'set-shell-file-name
+                (lambda* (#:key inputs #:allow-other-keys)
+                  ;; Refer to the right shell.
+                  (let ((bash (assoc-ref inputs "bash")))
+                    (substitute* "io.c"
+                      (("/bin/sh")
+                       (string-append bash "/bin/bash")))))
+                %standard-phases)))
    (inputs `(("libsigsegv" ,libsigsegv)))
    (home-page "http://www.gnu.org/software/gawk/")
-   (synopsis "GNU implementation of the Awk programming language")
+   (synopsis "A text scanning and processing language")
    (description
     "Many computer users need to manipulate text files: extract and then
 operate on data from parts of certain lines while discarding the rest, make
