@@ -38,9 +38,7 @@
     (build-system gnu-build-system)
     (inputs `(("ncurses" ,ncurses)))
     (home-page "https://www.gnu.org/software/less/")
-    (synopsis
-     "GNU less is a program similar to more, but which allows backward
-movement in the file as well as forward movement")
+    (synopsis "Paginator for terminals")
     (description
      "GNU less is a program similar to more, but which allows backward
 movement in the file as well as forward movement. Also, less does not
@@ -48,4 +46,4 @@ have to read the entire input file before starting, so with large input
 files it starts up faster than text editors like vi. Less uses
 termcap (or terminfo on some systems), so it can run on a variety of
 terminals. There is even limited support for hardcopy terminals.")
-    (license gpl3+))) ; some files are under GPLv2+
\ No newline at end of file
+    (license gpl3+))) ; some files are under GPLv2+
