@@ -27,7 +27,9 @@
   #:use-module (gnu packages bash)
   #:use-module (gnu packages compression)
   #:use-module (gnu packages gawk)
+  #:use-module (gnu packages gcc)
   #:use-module (gnu packages guile)
+  #:use-module (gnu packages bdw-gc)
   #:use-module (gnu packages linux)
   #:use-module (gnu packages multiprecision)
   #:use-module (ice-9 match)
@@ -52,17 +54,15 @@
   ;; without nscd, and with static NSS modules.
   (package (inherit glibc-final)
     (arguments
-     (lambda (system)
-       (substitute-keyword-arguments ((package-arguments glibc-final) system)
-         ((#:patches patches)
-          `(cons (assoc-ref %build-inputs "patch/system")
-                 ,patches))
-         ((#:configure-flags flags)
-          ;; Arrange so that getaddrinfo & co. do not contact the nscd,
-          ;; and can use statically-linked NSS modules.
-          `(cons* "--disable-nscd" "--disable-build-nscd"
-                  "--enable-static-nss"
-                  ,flags)))))
+     (substitute-keyword-arguments (package-arguments glibc-final)
+       ((#:patches patches)
+        `(cons (assoc-ref %build-inputs "patch/system") ,patches))
+       ((#:configure-flags flags)
+        ;; Arrange so that getaddrinfo & co. do not contact the nscd,
+        ;; and can use statically-linked NSS modules.
+        `(cons* "--disable-nscd" "--disable-build-nscd"
+                "--enable-static-nss"
+                ,flags))))
     (inputs
      `(("patch/system" ,(search-patch "glibc-bootstrap-system.patch"))
        ,@(package-inputs glibc-final)))))
@@ -119,19 +119,17 @@
                            %standard-phases)))))
         (gawk (package (inherit gawk)
                 (arguments
-                 (lambda (system)
-                   `(#:patches (list (assoc-ref %build-inputs "patch/sh"))
-                     ,@(substitute-keyword-arguments
-                           ((package-arguments gawk) system)
-                         ((#:phases phases)
-                          `(alist-cons-before
-                            'configure 'no-export-dynamic
-                            (lambda _
-                              ;; Since we use `-static', remove
-                              ;; `-export-dynamic'.
-                              (substitute* "configure"
-                                (("-export-dynamic") "")))
-                            ,phases))))))
+                 `(#:patches (list (assoc-ref %build-inputs "patch/sh"))
+                   ,@(substitute-keyword-arguments (package-arguments gawk)
+                       ((#:phases phases)
+                        `(alist-cons-before
+                          'configure 'no-export-dynamic
+                          (lambda _
+                            ;; Since we use `-static', remove
+                            ;; `-export-dynamic'.
+                            (substitute* "configure"
+                              (("-export-dynamic") "")))
+                          ,phases)))))
                 (inputs `(("patch/sh" ,(search-patch "gawk-shell.patch"))))))
         (finalize (lambda (p)
                     (static-package (package-with-explicit-inputs
@@ -253,7 +251,9 @@
 (define %binutils-static-stripped
   ;; The subset of Binutils that we need.
   (package (inherit %binutils-static)
+    (name (string-append (package-name %binutils-static) "-stripped"))
     (build-system trivial-build-system)
+    (outputs '("out"))
     (arguments
      `(#:modules ((guix build utils))
        #:builder
@@ -324,7 +324,10 @@
                                (string-append incdir "/asm-generic"))
              #t))))
       (inputs `(("libc" ,glibc)
-                ("linux-headers" ,linux-libre-headers))))))
+                ("linux-headers" ,linux-libre-headers)))
+
+      ;; Only one output.
+      (outputs '("out")))))
 
 (define %gcc-static
   ;; A statically-linked GCC, with stripped-down functionality.
@@ -332,29 +335,28 @@
    (package (inherit gcc-final)
      (name "gcc-static")
      (arguments
-      (lambda (system)
-        `(#:modules ((guix build utils)
-                     (guix build gnu-build-system)
-                     (srfi srfi-1)
-                     (srfi srfi-26)
-                     (ice-9 regex))
-          ,@(substitute-keyword-arguments ((package-arguments gcc-final) system)
-              ((#:guile _) #f)
-              ((#:implicit-inputs? _) #t)
-              ((#:configure-flags flags)
-               `(append (list
-                         "--disable-shared"
-                         "--disable-plugin"
-                         "--enable-languages=c"
-                         "--disable-libmudflap"
-                         "--disable-libgomp"
-                         "--disable-libssp"
-                         "--disable-libquadmath"
-                         "--disable-decimal-float")
-                        (remove (cut string-match "--(.*plugin|enable-languages)" <>)
-                                ,flags)))
-              ((#:make-flags flags)
-               `(cons "BOOT_LDFLAGS=-static" ,flags))))))
+      `(#:modules ((guix build utils)
+                   (guix build gnu-build-system)
+                   (srfi srfi-1)
+                   (srfi srfi-26)
+                   (ice-9 regex))
+        ,@(substitute-keyword-arguments (package-arguments gcc-final)
+            ((#:guile _) #f)
+            ((#:implicit-inputs? _) #t)
+            ((#:configure-flags flags)
+             `(append (list
+                       "--disable-shared"
+                       "--disable-plugin"
+                       "--enable-languages=c"
+                       "--disable-libmudflap"
+                       "--disable-libgomp"
+                       "--disable-libssp"
+                       "--disable-libquadmath"
+                       "--disable-decimal-float")
+                      (remove (cut string-match "--(.*plugin|enable-languages)" <>)
+                              ,flags)))
+            ((#:make-flags flags)
+             `(cons "BOOT_LDFLAGS=-static" ,flags)))))
      (inputs `(("gmp-source" ,(package-source gmp))
                ("mpfr-source" ,(package-source mpfr))
                ("mpc-source" ,(package-source mpc))
@@ -402,44 +404,58 @@
   ;; A statically-linked Guile that is relocatable--i.e., it can search
   ;; .scm and .go files relative to its installation directory, rather
   ;; than in hard-coded configure-time paths.
-  (let ((guile (package (inherit guile-2.0)
-                 (inputs
-                  `(("patch/relocatable"
-                     ,(search-patch "guile-relocatable.patch"))
-                    ("patch/utf8"
-                     ,(search-patch "guile-default-utf8.patch"))
-                    ,@(package-inputs guile-2.0)))
-                 (arguments
-                  `(;; When `configure' checks for ltdl availability, it
-                    ;; doesn't try to link using libtool, and thus fails
-                    ;; because of a missing -ldl.  Work around that.
-                    #:configure-flags '("LDFLAGS=-ldl")
+  (let* ((libgc (package (inherit libgc)
+                  (arguments
+                   ;; Make it so that we don't rely on /proc.  This is
+                   ;; especially useful in an initrd run before /proc is
+                   ;; mounted.
+                   '(#:configure-flags '("CPPFLAGS=-DUSE_LIBC_PRIVATES")))))
+         (guile (package (inherit guile-2.0)
+                  (name (string-append (package-name guile-2.0) "-static"))
+                  (inputs
+                   `(("patch/relocatable"
+                      ,(search-patch "guile-relocatable.patch"))
+                     ("patch/utf8"
+                      ,(search-patch "guile-default-utf8.patch"))
+                     ("patch/syscalls"
+                      ,(search-patch "guile-linux-syscalls.patch"))
+                     ,@(package-inputs guile-2.0)))
+                  (propagated-inputs
+                   `(("bdw-gc" ,libgc)
+                     ,@(alist-delete "bdw-gc"
+                                     (package-propagated-inputs guile-2.0))))
+                  (arguments
+                   `(;; When `configure' checks for ltdl availability, it
+                     ;; doesn't try to link using libtool, and thus fails
+                     ;; because of a missing -ldl.  Work around that.
+                     #:configure-flags '("LDFLAGS=-ldl")
 
-                    #:phases (alist-cons-before
-                              'configure 'static-guile
-                              (lambda _
-                                (substitute* "libguile/Makefile.in"
-                                  ;; Create a statically-linked `guile'
-                                  ;; executable.
-                                  (("^guile_LDFLAGS =")
-                                   "guile_LDFLAGS = -all-static")
+                     #:phases (alist-cons-before
+                               'configure 'static-guile
+                               (lambda _
+                                 (substitute* "libguile/Makefile.in"
+                                   ;; Create a statically-linked `guile'
+                                   ;; executable.
+                                   (("^guile_LDFLAGS =")
+                                    "guile_LDFLAGS = -all-static")
 
-                                  ;; Add `-ldl' *after* libguile-2.0.la.
-                                  (("^guile_LDADD =(.*)$" _ ldadd)
-                                   (string-append "guile_LDADD = "
-                                                  (string-trim-right ldadd)
-                                                  " -ldl\n"))))
-                              %standard-phases)
+                                   ;; Add `-ldl' *after* libguile-2.0.la.
+                                   (("^guile_LDADD =(.*)$" _ ldadd)
+                                    (string-append "guile_LDADD = "
+                                                   (string-trim-right ldadd)
+                                                   " -ldl\n"))))
+                               %standard-phases)
 
-                    ;; Allow Guile to be relocated, as is needed during
-                    ;; bootstrap.
-                    #:patches
-                    (list (assoc-ref %build-inputs "patch/relocatable")
-                          (assoc-ref %build-inputs "patch/utf8"))
+                     ;; Allow Guile to be relocated, as is needed during
+                     ;; bootstrap.
+                     #:patches
+                     (list (assoc-ref %build-inputs "patch/relocatable")
+                           (assoc-ref %build-inputs "patch/utf8")
+                           (assoc-ref %build-inputs "patch/syscalls"))
 
-                    ;; There are uses of `dynamic-link' in
-                    ;; {foreign,coverage}.test that don't fly here.
-                    #:tests? #f)))))
+                     ;; There are uses of `dynamic-link' in
+                     ;; {foreign,coverage}.test that don't fly here.
+                     #:tests? #f)))))
     (package-with-explicit-inputs (static-package guile)
                                   %standard-inputs-with-relocatable-glibc
                                   (current-source-location))))
@@ -482,25 +498,25 @@
               ("xz" ,xz)
               ("input" ,pkg)))
     (arguments
-     (lambda (system)
-       (let ((name    (package-name pkg))
-             (version (package-version pkg)))
-         `(#:modules ((guix build utils))
-           #:builder
-           (begin
-             (use-modules (guix build utils))
-             (let ((out   (assoc-ref %outputs "out"))
-                   (input (assoc-ref %build-inputs "input"))
-                   (tar   (assoc-ref %build-inputs "tar"))
-                   (xz    (assoc-ref %build-inputs "xz")))
-               (mkdir out)
-               (set-path-environment-variable "PATH" '("bin") (list tar xz))
-               (with-directory-excursion input
-                 (zero? (system* "tar" "cJvf"
-                                 (string-append out "/"
-                                                ,name "-" ,version
-                                                "-" ,system ".tar.xz")
-                                 ".")))))))))))
+     (let ((name    (package-name pkg))
+           (version (package-version pkg)))
+       `(#:modules ((guix build utils))
+         #:builder
+         (begin
+           (use-modules (guix build utils))
+           (let ((out   (assoc-ref %outputs "out"))
+                 (input (assoc-ref %build-inputs "input"))
+                 (tar   (assoc-ref %build-inputs "tar"))
+                 (xz    (assoc-ref %build-inputs "xz")))
+             (mkdir out)
+             (set-path-environment-variable "PATH" '("bin") (list tar xz))
+             (with-directory-excursion input
+               (zero? (system* "tar" "cJvf"
+                               (string-append out "/"
+                                              ,name "-" ,version
+                                              "-" ,(%current-system)
+                                              ".tar.xz")
+                               "."))))))))))
 
 (define %bootstrap-binaries-tarball
   ;; A tarball with the statically-linked bootstrap binaries.
