@@ -1,5 +1,5 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2012 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2012, 2013 Ludovic Courtès <ludo@gnu.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -21,13 +21,14 @@
   #:use-module (guix utils)
   #:use-module (guix derivations)
   #:use-module (guix build-system)
-  #:use-module (guix build-system gnu)
   #:use-module (guix packages)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-39)
   #:use-module (ice-9 match)
   #:export (gnu-build
             gnu-build-system
+            standard-search-paths
+            standard-inputs
             package-with-explicit-inputs
             package-with-extra-configure-variable
             static-libgcc-package
@@ -66,12 +67,8 @@ when GUILE is #f."
     (location (if (pair? loc) (source-properties->location loc) loc))
     (arguments
      (let ((args (package-arguments p)))
-       (if (procedure? args)
-           (lambda (system)
-             `(#:guile ,guile
-               #:implicit-inputs? #f ,@(args system)))
-           `(#:guile ,guile
-             #:implicit-inputs? #f ,@args))))
+       `(#:guile ,guile
+         #:implicit-inputs? #f ,@args)))
     (native-inputs (map rewritten-input
                         (filtered-inputs (package-native-inputs p))))
     (propagated-inputs (map rewritten-input
@@ -82,9 +79,9 @@ when GUILE is #f."
                      (filtered-inputs (package-inputs p)))))))
 
 (define (package-with-extra-configure-variable p variable value)
-  "Return a version of P with VARIABLE=VALUE specified as an extra
-`configure' flag.  An example is LDFLAGS=-static.  If P already has
-configure flags for VARIABLE, the associated value is augmented."
+  "Return a version of P with VARIABLE=VALUE specified as an extra `configure'
+flag, recursively.  An example is LDFLAGS=-static.  If P already has configure
+flags for VARIABLE, the associated value is augmented."
   (let loop ((p p))
     (define (rewritten-inputs inputs)
       (map (match-lambda
@@ -95,23 +92,19 @@ configure flags for VARIABLE, the associated value is augmented."
 
     (package (inherit p)
       (arguments
-       (lambda (system)
-         (let ((args (match (package-arguments p)
-                       ((? procedure? proc)
-                        (proc system))
-                       (x x))))
-          (substitute-keyword-arguments args
-            ((#:configure-flags flags)
-             (let* ((var= (string-append variable "="))
-                    (len  (string-length var=)))
-               `(cons ,(string-append var= value)
-                      (map (lambda (flag)
-                             (if (string-prefix? ,var= flag)
-                                 (string-append
-                                  ,(string-append var= value " ")
-                                  (substring flag ,len))
-                                 flag))
-                           ,flags))))))))
+       (let ((args (package-arguments p)))
+         (substitute-keyword-arguments args
+           ((#:configure-flags flags)
+            (let* ((var= (string-append variable "="))
+                   (len  (string-length var=)))
+              `(cons ,(string-append var= value)
+                     (map (lambda (flag)
+                            (if (string-prefix? ,var= flag)
+                                (string-append
+                                 ,(string-append var= value " ")
+                                 (substring flag ,len))
+                                flag))
+                          ,flags)))))))
       (inputs (rewritten-inputs (package-inputs p)))
       (propagated-inputs (rewritten-inputs (package-propagated-inputs p))))))
 
@@ -119,33 +112,48 @@ configure flags for VARIABLE, the associated value is augmented."
   "A version of P linked with `-static-gcc'."
   (package-with-extra-configure-variable p "LDFLAGS" "-static-libgcc"))
 
-(define* (static-package p #:optional (loc (current-source-location)))
-  "Return a statically-linked version of package P."
+(define* (static-package p #:optional (loc (current-source-location))
+                         #:key (strip-all? #t))
+  "Return a statically-linked version of package P.  If STRIP-ALL? is true,
+use `--strip-all' as the arguments to `strip'."
   (let ((args (package-arguments p)))
     (package (inherit p)
       (location (source-properties->location loc))
       (arguments
-       (let ((augment (lambda (args)
-                        (let ((a (default-keyword-arguments args
-                                   '(#:configure-flags '()
-                                     #:strip-flags #f))))
-                          (substitute-keyword-arguments a
-                            ((#:configure-flags flags)
-                             `(cons* "--disable-shared"
-                                     "LDFLAGS=-static"
-                                     ,flags))
-                            ((#:strip-flags _)
-                             ''("--strip-all")))))))
-         (if (procedure? args)
-             (lambda x
-               (augment (apply args x)))
-             (augment args)))))))
+       (let ((a (default-keyword-arguments args
+                  '(#:configure-flags '()
+                    #:strip-flags '("--strip-debug")))))
+         (substitute-keyword-arguments a
+           ((#:configure-flags flags)
+            `(cons* "--disable-shared" "LDFLAGS=-static" ,flags))
+           ((#:strip-flags flags)
+            (if strip-all?
+                ''("--strip-all")
+                flags))))))))
 
 
 (define %store
   ;; Store passed to STANDARD-INPUTS.
   (make-parameter #f))
 
+(define (standard-packages)
+  "Return the list of (NAME PACKAGE OUTPUT) or (NAME PACKAGE) tuples of
+standard packages used as implicit inputs of the GNU build system."
+
+  ;; Resolve (gnu packages base) lazily to hide circular dependency.
+  (let ((distro (resolve-module '(gnu packages base))))
+    (module-ref distro '%final-inputs)))
+
+(define (standard-search-paths)
+  "Return the list of <search-path-specification> for the standard (implicit)
+inputs."
+  (append-map (match-lambda
+               ((_ (? package? p) _ ...)
+                (package-native-search-paths p))
+               (_
+                '()))
+              (standard-packages)))
+
 (define standard-inputs
   (memoize
    (lambda (system)
@@ -159,9 +167,7 @@ System: GCC, GNU Make, Bash, Coreutils, etc."
            (z
             (error "invalid standard input" z)))
 
-          ;; Resolve (gnu packages base) lazily to hide circular dependency.
-          (let* ((distro (resolve-module '(gnu packages base)))
-                 (inputs (module-ref distro '%final-inputs)))
+          (let ((inputs (standard-packages)))
             (append inputs
                     (append-map (match-lambda
                                  ((name package _ ...)
@@ -170,11 +176,12 @@ System: GCC, GNU Make, Bash, Coreutils, etc."
 
 (define* (gnu-build store name source inputs
                     #:key (guile #f)
-                    (outputs '("out")) (configure-flags ''())
+                    (outputs '("out"))
+                    (search-paths '())
+                    (configure-flags ''())
                     (make-flags ''())
                     (patches ''()) (patch-flags ''("--batch" "-p1"))
                     (out-of-source? #f)
-                    (path-exclusions ''())
                     (tests? #t)
                     (test-target "check")
                     (parallel-build? #t) (parallel-tests? #t)
@@ -201,6 +208,16 @@ the builder's environment, from the host.  Note that we distinguish
 between both, because for Guile's own modules like (ice-9 foo), we want
 to use GUILE's own version of it, rather than import the user's one,
 which could lead to gratuitous input divergence."
+  (define implicit-inputs
+    (and implicit-inputs?
+         (parameterize ((%store store))
+           (standard-inputs system))))
+
+  (define implicit-search-paths
+    (if implicit-inputs?
+        (standard-search-paths)
+        '()))
+
   (define builder
     `(begin
        (use-modules ,@modules)
@@ -210,13 +227,15 @@ which could lead to gratuitous input divergence."
                   #:system ,system
                   #:outputs %outputs
                   #:inputs %build-inputs
+                  #:search-paths ',(map search-path-specification->sexp
+                                        (append implicit-search-paths
+                                                search-paths))
                   #:patches ,patches
                   #:patch-flags ,patch-flags
                   #:phases ,phases
                   #:configure-flags ,configure-flags
                   #:make-flags ,make-flags
                   #:out-of-source? ,out-of-source?
-                  #:path-exclusions ,path-exclusions
                   #:tests? ,tests?
                   #:test-target ,test-target
                   #:parallel-build? ,parallel-build?
@@ -244,8 +263,7 @@ which could lead to gratuitous input divergence."
                                         '())
                                   ,@inputs
                                   ,@(if implicit-inputs?
-                                        (parameterize ((%store store))
-                                          (standard-inputs system))
+                                        implicit-inputs
                                         '()))
                                 #:outputs outputs
                                 #:modules imported-modules
