@@ -17,8 +17,10 @@
 ;;; along with GNU Guix.  If not, see <http://www.gnu.org/licenses/>.
 
 (define-module (guix store)
+  #:use-module (guix nar)
   #:use-module (guix utils)
   #:use-module (guix config)
+  #:use-module (guix serialization)
   #:use-module (rnrs bytevectors)
   #:use-module (rnrs io ports)
   #:use-module (srfi srfi-1)
@@ -28,15 +30,18 @@
   #:use-module (srfi srfi-35)
   #:use-module (srfi srfi-39)
   #:use-module (ice-9 match)
-  #:use-module (ice-9 rdelim)
-  #:use-module (ice-9 ftw)
   #:use-module (ice-9 regex)
-  #:export (nix-server?
+  #:export (%daemon-socket-file
+
+            nix-server?
             nix-server-major-version
             nix-server-minor-version
             nix-server-socket
 
             &nix-error nix-error?
+            &nix-connection-error nix-connection-error?
+            nix-connection-error-file
+            nix-connection-error-code
             &nix-protocol-error nix-protocol-error?
             nix-protocol-error-message
             nix-protocol-error-status
@@ -54,6 +59,20 @@
             add-temp-root
             add-indirect-root
 
+            substitutable?
+            substitutable-path
+            substitutable-deriver
+            substitutable-references
+            substitutable-download-size
+            substitutable-nar-size
+            has-substitutes?
+            substitutable-paths
+            substitutable-path-info
+
+            references
+            referrers
+            valid-derivers
+            query-derivation-outputs
             live-paths
             dead-paths
             collect-garbage
@@ -64,9 +83,10 @@
             %store-prefix
             store-path?
             derivation-path?
-            store-path-package-name))
+            store-path-package-name
+            store-path-hash-part))
 
-(define %protocol-version #x10b)
+(define %protocol-version #x10c)
 
 (define %worker-magic-1 #x6e697863)
 (define %worker-magic-2 #x6478696f)
@@ -103,14 +123,19 @@
   (query-deriver 18)
   (set-options 19)
   (collect-garbage 20)
-  (query-substitutable-path-info 21)
+  ;;(query-substitutable-path-info 21)  ; obsolete as of #x10c
   (query-derivation-outputs 22)
-  (query-valid-paths 23)
+  (query-all-valid-paths 23)
   (query-failed-paths 24)
   (clear-failed-paths 25)
   (query-path-info 26)
   (import-paths 27)
-  (query-derivation-output-names 28))
+  (query-derivation-output-names 28)
+  (query-path-from-hash-part 29)
+  (query-substitutable-path-infos 30)
+  (query-valid-paths 31)
+  (query-substitutable-paths 32)
+  (query-valid-derivers 33))
 
 (define-enumerate-type hash-algo
   ;; hash.hh
@@ -129,129 +154,36 @@
   (string-append (or (getenv "NIX_STATE_DIR") %state-directory)
                  "/daemon-socket/socket"))
 
+(define %daemon-socket-file
+  ;; File name of the socket the daemon listens too.
+  (make-parameter (or (getenv "GUIX_DAEMON_SOCKET")
+                      %default-socket-path)))
+
+
 
-;; serialize.cc
+;; Information about a substitutable store path.
+(define-record-type <substitutable>
+  (substitutable path deriver refs dl-size nar-size)
+  substitutable?
+  (path      substitutable-path)
+  (deriver   substitutable-deriver)
+  (refs      substitutable-references)
+  (dl-size   substitutable-download-size)
+  (nar-size  substitutable-nar-size))
 
-(define (write-int n p)
-  (let ((b (make-bytevector 8 0)))
-    (bytevector-u32-set! b 0 n (endianness little))
-    (put-bytevector p b)))
-
-(define (read-int p)
-  (let ((b (get-bytevector-n p 8)))
-    (bytevector-u32-ref b 0 (endianness little))))
-
-(define (write-long-long n p)
-  (let ((b (make-bytevector 8 0)))
-    (bytevector-u64-set! b 0 n (endianness little))
-    (put-bytevector p b)))
-
-(define (read-long-long p)
-  (let ((b (get-bytevector-n p 8)))
-    (bytevector-u64-ref b 0 (endianness little))))
-
-(define write-padding
-  (let ((zero (make-bytevector 8 0)))
-    (lambda (n p)
-      (let ((m (modulo n 8)))
-        (or (zero? m)
-            (put-bytevector p zero 0 (- 8 m)))))))
-
-(define (write-string s p)
-  (let ((b (string->utf8 s)))
-    (write-int (bytevector-length b) p)
-    (put-bytevector p b)
-    (write-padding (bytevector-length b) p)))
-
-(define (read-string p)
-  (let* ((len (read-int p))
-         (m   (modulo len 8))
-         (bv  (get-bytevector-n p len))
-         (str (utf8->string bv)))
-    (or (zero? m)
-        (get-bytevector-n p (- 8 m)))
-    str))
-
-(define (write-string-list l p)
-  (write-int (length l) p)
-  (for-each (cut write-string <> p) l))
-
-(define (read-string-list p)
-  (let ((len (read-int p)))
-    (unfold (cut >= <> len)
-            (lambda (i)
-              (read-string p))
-            1+
-            0)))
-
-(define (write-store-path f p)
-  (write-string f p))                             ; TODO: assert path
-
-(define (read-store-path p)
-  (read-string p))                                ; TODO: assert path
-
-(define write-store-path-list write-string-list)
-(define read-store-path-list read-string-list)
-
-(define (write-contents file p)
-  "Write the contents of FILE to output port P."
-  (define (dump in size)
-    (define buf-size 65536)
-    (define buf (make-bytevector buf-size))
-
-    (let loop ((left size))
-      (if (<= left 0)
-          0
-          (let ((read (get-bytevector-n! in buf 0 buf-size)))
-            (if (eof-object? read)
-                left
-                (begin
-                  (put-bytevector p buf 0 read)
-                  (loop (- left read))))))))
-
-  (let ((size (stat:size (lstat file))))
-    (write-string "contents" p)
-    (write-long-long size p)
-    (call-with-input-file file
-      (lambda (p)
-        (dump p size)))
-    (write-padding size p)))
-
-(define (write-file f p)
-  (define %archive-version-1 "nix-archive-1")
-
-  (write-string %archive-version-1 p)
-
-  (let dump ((f f))
-    (let ((s (lstat f)))
-      (write-string "(" p)
-      (case (stat:type s)
-        ((regular)
-         (write-string "type" p)
-         (write-string "regular" p)
-         (if (not (zero? (logand (stat:mode s) #o100)))
-             (begin
-               (write-string "executable" p)
-               (write-string "" p)))
-         (write-contents f p))
-        ((directory)
-         (write-string "type" p)
-         (write-string "directory" p)
-         (let ((entries (remove (cut member <> '("." ".."))
-                                (scandir f))))
-           (for-each (lambda (e)
-                       (let ((f (string-append f "/" e)))
-                         (write-string "entry" p)
-                         (write-string "(" p)
-                         (write-string "name" p)
-                         (write-string e p)
-                         (write-string "node" p)
-                         (dump f)
-                         (write-string ")" p)))
-                     entries)))
-        (else
-         (error "ENOSYS")))
-      (write-string ")" p))))
+(define (read-substitutable-path-list p)
+  (let loop ((len    (read-int p))
+             (result '()))
+    (if (zero? len)
+        (reverse result)
+        (let ((path     (read-store-path p))
+              (deriver  (read-store-path p))
+              (refs     (read-store-path-list p))
+              (dl-size  (read-long-long p))
+              (nar-size (read-long-long p)))
+          (loop (- len 1)
+                (cons (substitutable path deriver refs dl-size nar-size)
+                      result))))))
 
 (define-syntax write-arg
   (syntax-rules (integer boolean file string string-list
@@ -274,7 +206,8 @@
      (write-string (bytevector->base16-string arg) p))))
 
 (define-syntax read-arg
-  (syntax-rules (integer boolean string store-path store-path-list base16)
+  (syntax-rules (integer boolean string store-path store-path-list
+                 substitutable-path-list base16)
     ((_ integer p)
      (read-int p))
     ((_ boolean p)
@@ -285,28 +218,42 @@
      (read-store-path p))
     ((_ store-path-list p)
      (read-store-path-list p))
-    ((_ hash p)
+    ((_ substitutable-path-list p)
+     (read-substitutable-path-list p))
+    ((_ base16 p)
      (base16-string->bytevector (read-string p)))))
 
 
 ;; remote-store.cc
 
 (define-record-type <nix-server>
-  (%make-nix-server socket major minor)
+  (%make-nix-server socket major minor
+                    ats-cache atts-cache)
   nix-server?
   (socket nix-server-socket)
   (major  nix-server-major-version)
-  (minor  nix-server-minor-version))
+  (minor  nix-server-minor-version)
+
+  ;; Caches.  We keep them per-connection, because store paths build
+  ;; during the session are temporary GC roots kept for the duration of
+  ;; the session.
+  (ats-cache  nix-server-add-to-store-cache)
+  (atts-cache nix-server-add-text-to-store-cache))
 
 (define-condition-type &nix-error &error
   nix-error?)
 
+(define-condition-type &nix-connection-error &nix-error
+  nix-connection-error?
+  (file   nix-connection-error-file)
+  (errno  nix-connection-error-code))
+
 (define-condition-type &nix-protocol-error &nix-error
   nix-protocol-error?
   (message nix-protocol-error-message)
   (status  nix-protocol-error-status))
 
-(define* (open-connection #:optional (file %default-socket-path)
+(define* (open-connection #:optional (file (%daemon-socket-file))
                           #:key (reserve-space? #t))
   "Connect to the daemon over the Unix-domain socket at FILE.  When
 RESERVE-SPACE? is true, instruct it to reserve a little bit of extra
@@ -320,7 +267,15 @@ operate, should the disk become full.  Return a server object."
     ;; Enlarge the receive buffer.
     (setsockopt s SOL_SOCKET SO_RCVBUF (* 12 1024))
 
-    (connect s a)
+    (catch 'system-error
+      (cut connect s a)
+      (lambda args
+        ;; Translate the error to something user-friendly.
+        (let ((errno (system-error-errno args)))
+          (raise (condition (&nix-connection-error
+                             (file file)
+                             (errno errno)))))))
+
     (write-int %worker-magic-1 s)
     (let ((r (read-int s)))
       (and (eqv? r %worker-magic-2)
@@ -333,7 +288,9 @@ operate, should the disk become full.  Return a server object."
                         (write-int (if reserve-space? 1 0) s))
                     (let ((s (%make-nix-server s
                                                (protocol-major v)
-                                               (protocol-minor v))))
+                                               (protocol-minor v)
+                                               (make-hash-table 100)
+                                               (make-hash-table 100))))
                       (let loop ((done? (process-stderr s)))
                         (or done? (process-stderr s)))
                       s))))))))
@@ -350,7 +307,11 @@ operate, should the disk become full.  Return a server object."
   "Read standard output and standard error from SERVER, writing it to
 CURRENT-BUILD-OUTPUT-PORT.  Return #t when SERVER is done sending data, and
 #f otherwise; in the latter case, the caller should call `process-stderr'
-again until #t is returned or an error is raised."
+again until #t is returned or an error is raised.
+
+Since the build process's output cannot be assumed to be UTF-8, we
+conservatively consider it to be Latin-1, thereby avoiding possible
+encoding conversion errors."
   (define p
     (nix-server-socket server))
 
@@ -363,19 +324,22 @@ again until #t is returned or an error is raised."
 
   (let ((k (read-int p)))
     (cond ((= k %stderr-write)
-           (read-string p)
+           (read-latin1-string p)
            #f)
           ((= k %stderr-read)
            (let ((len (read-int p)))
-             (read-string p)                      ; FIXME: what to do?
+             (read-latin1-string p)               ; FIXME: what to do?
              #f))
           ((= k %stderr-next)
-           (let ((s (read-string p)))
+           (let ((s (read-latin1-string p)))
              (display s (current-build-output-port))
              #f))
           ((= k %stderr-error)
-           (let ((error  (read-string p))
-                 (status (if (>= (nix-server-minor-version server) 8)
+           (let ((error  (read-latin1-string p))
+                 ;; Currently the daemon fails to send a status code for early
+                 ;; errors like DB schema version mismatches, so check for EOF.
+                 (status (if (and (>= (nix-server-minor-version server) 8)
+                                  (not (eof-object? (lookahead-u8 p))))
                              (read-int p)
                              1)))
              (raise (condition (&nix-protocol-error
@@ -399,7 +363,8 @@ again until #t is returned or an error is raised."
                             (log-type 0)
                             (print-build-trace #t)
                             (build-cores 1)
-                            (use-substitutes? #t))
+                            (use-substitutes? #t)
+                            (binary-caches '())) ; client "untrusted" cache URLs
   ;; Must be called after `open-connection'.
 
   (define socket
@@ -423,13 +388,21 @@ again until #t is returned or an error is raised."
         (send (integer build-cores)))
     (if (>= (nix-server-minor-version server) 10)
         (send (boolean use-substitutes?)))
+    (if (>= (nix-server-minor-version server) 12)
+        (send (string-list (fold-right (lambda (pair result)
+                                         (match pair
+                                           ((h . t)
+                                            (cons* h t result))))
+                                       '()
+                                       binary-caches))))
     (let loop ((done? (process-stderr server)))
       (or done? (process-stderr server)))))
 
-(define-syntax define-operation
+(define-syntax operation
   (syntax-rules ()
+    "Define a client-side RPC stub for the given operation."
     ((_ (name (type arg) ...) docstring return ...)
-     (define (name server arg ...)
+     (lambda (server arg ...)
        docstring
        (let ((s (nix-server-socket server)))
          (write-int (operation-id name) s)
@@ -440,29 +413,63 @@ again until #t is returned or an error is raised."
            (or done? (loop (process-stderr server))))
          (values (read-arg return s) ...))))))
 
+(define-syntax-rule (define-operation (name args ...)
+                      docstring return ...)
+  (define name
+    (operation (name args ...) docstring return ...)))
+
 (define-operation (valid-path? (string path))
   "Return #t when PATH is a valid store path."
   boolean)
 
-(define-operation (query-path-hash (string path))
+(define-operation (query-path-hash (store-path path))
   "Return the SHA256 hash of PATH as a bytevector."
   base16)
 
-(define-operation (add-text-to-store (string name) (string text)
-                                     (string-list references))
-  "Add TEXT under file NAME in the store, and return its store path.
+(define add-text-to-store
+  ;; A memoizing version of `add-to-store', to avoid repeated RPCs with
+  ;; the very same arguments during a given session.
+  (let ((add-text-to-store
+         (operation (add-text-to-store (string name) (string text)
+                                       (string-list references))
+                    #f
+                    store-path)))
+    (lambda (server name text references)
+      "Add TEXT under file NAME in the store, and return its store path.
 REFERENCES is the list of store paths referred to by the resulting store
 path."
-  store-path)
+      (let ((args  `(,text ,name ,references))
+            (cache (nix-server-add-text-to-store-cache server)))
+        (or (hash-ref cache args)
+            (let ((path (add-text-to-store server name text references)))
+              (hash-set! cache args path)
+              path))))))
 
-(define-operation (add-to-store (string basename)
-                                (boolean fixed?)  ; obsolete, must be #t
-                                (boolean recursive?)
-                                (string hash-algo)
-                                (file file-name))
-  "Add the contents of FILE-NAME under BASENAME to the store.  Note that
-FIXED? is for backward compatibility with old Nix versions and must be #t."
-  store-path)
+(define add-to-store
+  ;; A memoizing version of `add-to-store'.  This is important because
+  ;; `add-to-store' leads to huge data transfers to the server, and
+  ;; because it's often called many times with the very same argument.
+  (let ((add-to-store (operation (add-to-store (string basename)
+                                               (boolean fixed?) ; obsolete, must be #t
+                                               (boolean recursive?)
+                                               (string hash-algo)
+                                               (file file-name))
+                                 #f
+                                 store-path)))
+    (lambda (server basename recursive? hash-algo file-name)
+      "Add the contents of FILE-NAME under BASENAME to the store.  When
+RECURSIVE? is true and FILE-NAME designates a directory, the contents of
+FILE-NAME are added recursively; if FILE-NAME designates a flat file and
+RECURSIVE? is true, its contents are added, and its permission bits are
+kept.  HASH-ALGO must be a string such as \"sha256\"."
+      (let* ((st    (stat file-name #f))
+             (args  `(,st ,basename ,recursive? ,hash-algo))
+             (cache (nix-server-add-to-store-cache server)))
+        (or (and st (hash-ref cache args))
+            (let ((path (add-to-store server basename #t recursive?
+                                      hash-algo file-name)))
+              (hash-set! cache args path)
+              path))))))
 
 (define-operation (build-derivations (string-list derivations))
   "Build DERIVATIONS, and return when the worker is done building them.
@@ -481,6 +488,43 @@ name--it is the caller's responsibility to ensure that it is an absolute
 file name.  Return #t on success."
   boolean)
 
+(define references
+  (operation (query-references (store-path path))
+             "Return the list of references of PATH."
+             store-path-list))
+
+(define referrers
+  (operation (query-referrers (store-path path))
+             "Return the list of path that refer to PATH."
+             store-path-list))
+
+(define valid-derivers
+  (operation (query-valid-derivers (store-path path))
+             "Return the list of valid \"derivers\" of PATH---i.e., all the
+.drv present in the store that have PATH among their outputs."
+             store-path-list))
+
+(define query-derivation-outputs  ; avoid name clash with `derivation-outputs'
+  (operation (query-derivation-outputs (store-path path))
+             "Return the list of outputs of PATH, a .drv file."
+             store-path-list))
+
+(define-operation (has-substitutes? (store-path path))
+  "Return #t if binary substitutes are available for PATH, and #f otherwise."
+  boolean)
+
+(define substitutable-paths
+  (operation (query-substitutable-paths (store-path-list paths))
+             "Return the subset of PATHS that is substitutable."
+             store-path-list))
+
+(define substitutable-path-info
+  (operation (query-substitutable-path-infos (store-path-list paths))
+             "Return information about the subset of PATHS that is
+substitutable.  For each substitutable path, a `substitutable?' object is
+returned."
+             substitutable-path-list))
+
 (define (run-gc server action to-delete min-freed)
   "Perform the garbage-collector operation ACTION, one of the
 `gc-action' values.  When ACTION is `delete-specific', the TO-DELETE is
@@ -565,3 +609,12 @@ collected, and the number of bytes freed."
 
   (and=> (regexp-exec store-path-rx path)
          (cut match:substring <> 1)))
+
+(define (store-path-hash-part path)
+  "Return the hash part of PATH as a base32 string, or #f if PATH is not a
+syntactically valid store path."
+  (let ((path-rx (make-regexp
+                  (string-append"^" (regexp-quote (%store-prefix))
+                                "/([0-9a-df-np-sv-z]{32})-[^/]+$"))))
+    (and=> (regexp-exec path-rx path)
+           (cut match:substring <> 1))))
