@@ -32,6 +32,7 @@
   #:use-module (srfi srfi-64)
   #:use-module (rnrs io ports)
   #:use-module (rnrs bytevectors)
+  #:use-module (web uri)
   #:use-module (ice-9 rdelim)
   #:use-module (ice-9 regex)
   #:use-module (ice-9 ftw)
@@ -51,7 +52,7 @@
 (define %bash
   (let ((bash (search-bootstrap-binary "bash" (%current-system))))
     (and %store
-         (add-to-store %store "bash" #t #t "sha256" bash))))
+         (add-to-store %store "bash" #t "sha256" bash))))
 
 (define (directory-contents dir)
   "Return an alist representing the contents of DIR."
@@ -86,7 +87,7 @@
 
 (test-assert "add-to-store, flat"
   (let* ((file (search-path %load-path "language/tree-il/spec.scm"))
-         (drv  (add-to-store %store "flat-test" #t #f "sha256" file)))
+         (drv  (add-to-store %store "flat-test" #f "sha256" file)))
     (and (eq? 'regular (stat:type (stat drv)))
          (valid-path? %store drv)
          (equal? (call-with-input-file file get-bytevector-all)
@@ -94,7 +95,7 @@
 
 (test-assert "add-to-store, recursive"
   (let* ((dir (dirname (search-path %load-path "language/tree-il/spec.scm")))
-         (drv (add-to-store %store "dir-tree-test" #t #t "sha256" dir)))
+         (drv (add-to-store %store "dir-tree-test" #t "sha256" dir)))
     (and (eq? 'directory (stat:type (stat drv)))
          (valid-path? %store drv)
          (equal? (directory-contents dir)
@@ -134,7 +135,7 @@
 (test-assert "derivation with local file as input"
   (let* ((builder    (add-text-to-store
                       %store "my-builder.sh"
-                      "(while read line ; do echo $line ; done) < $in > $out"
+                      "(while read line ; do echo \"$line\" ; done) < $in > $out"
                       '()))
          (input      (search-path %load-path "ice-9/boot-9.scm"))
          (drv-path   (derivation %store "derivation-with-input-file"
@@ -145,14 +146,14 @@
                                     ;; name to the builder.
                                     . ,(add-to-store %store
                                                      (basename input)
-                                                     #t #t "sha256"
+                                                     #t "sha256"
                                                      input)))
                                  `((,builder)
                                    (,input)))))   ; ← local file name
     (and (build-derivations %store (list drv-path))
-         (let ((p (derivation-path->output-path drv-path)))
-           (and (call-with-input-file p get-bytevector-all)
-                (call-with-input-file input get-bytevector-all))))))
+         ;; Note: we can't compare the files because the above trick alters
+         ;; the contents.
+         (valid-path? %store (derivation-path->output-path drv-path)))))
 
 (test-assert "fixed-output derivation"
   (let* ((builder    (add-text-to-store %store "my-fixed-builder.sh"
@@ -398,6 +399,51 @@
          ;; prerequisite to build because DRV itself is already built.
          (null? (derivation-prerequisites-to-build %store drv)))))
 
+(test-skip (if (getenv "GUIX_BINARY_SUBSTITUTE_URL") 0 1))
+(test-assert "derivation-prerequisites-to-build and substitutes"
+  (let*-values (((store)
+                 (open-connection))
+                ((drv-path drv)
+                 (build-expression->derivation store "prereq-subst"
+                                               (%current-system)
+                                               (random 1000) '()))
+                ((output)
+                 (derivation-output-path
+                  (assoc-ref (derivation-outputs drv) "out")))
+                ((dir)
+                 (and=> (getenv "GUIX_BINARY_SUBSTITUTE_URL")
+                        (compose uri-path string->uri))))
+    ;; Create fake substituter data, to be read by `substitute-binary'.
+    (call-with-output-file (string-append dir "/nix-cache-info")
+      (lambda (p)
+        (format p "StoreDir: ~a\nWantMassQuery: 0\n"
+                (%store-prefix))))
+    (call-with-output-file (string-append dir "/" (store-path-hash-part output)
+                                          ".narinfo")
+      (lambda (p)
+        (format p "StorePath: ~a
+URL: ~a
+Compression: none
+NarSize: 1234
+References: 
+System: ~a
+Deriver: ~a~%"
+                output                              ; StorePath
+                (string-append dir "/example.nar")  ; URL
+                (%current-system)                   ; System
+                (basename drv-path))))              ; Deriver
+
+    (let-values (((build download)
+                  (derivation-prerequisites-to-build store drv))
+                 ((build* download*)
+                  (derivation-prerequisites-to-build store drv
+                                                     #:use-substitutes? #f)))
+      (pk build download build* download*)
+      (and (null? build)
+           (equal? download (list output))
+           (null? download*)
+           (null? build*)))))
+
 (test-assert "build-expression->derivation with expression returning #f"
   (let* ((builder  '(begin
                       (mkdir %output)
@@ -547,8 +593,3 @@
 
 
 (exit (= (test-runner-fail-count (test-runner-current)) 0))
-
-;;; Local Variables:
-;;; eval: (put 'test-assert 'scheme-indent-function 1)
-;;; eval: (put 'guard 'scheme-indent-function 1)
-;;; End:
